import java.io.*;
import java.math.*;
import java.util.*;

public class TestGen {
    int testCount, subtaskCount;
    PrintWriter testsDesc;

    Random random = new Random(787788);

    final int MAX_N = 200_000;

    String[] samplesString = new String[]{"WWW", "BBBBBB"};
    int[][] samplesChanges = new int[][]{{1, 2, 3, 2}, {1, 3, 5, 2, 4, 6}};
    String[] handString = new String[]{"WWBB", "WBWBWBWBWB"};
    int[][] handChanges = new int[][]{{1, 3, 4, 3, 1, 3, 2}, {1, 3, 5, 7, 9, 1, 1, 1, 1, 1}};

    void write(String s, int[] c, String folder, String subtask, int points, String comment) throws IOException {
        String path = String.format("../%s/%02d", folder, ++testCount);
        PrintWriter out = new PrintWriter(path);
        out.println(s.length() + " " + c.length);
        out.println(s);
        for (int x : c) {
            out.println(x);
        }
        out.close();
        testsDesc.println(String.format("%s, folder=%s, subtask=%s, points=%d, comment=%s", path, folder, subtask, points, comment));
        System.out.println(String.format("Test %d is generated", testCount));
    }

    void genPreliminary() throws IOException {
        for (int i = 0; i < samplesString.length; i++) {
            write(samplesString[i], samplesChanges[i], "preliminary", "subtask0", 0, String.format("\"sample %d\"", i + 1));
        }
        testCount = 0;
    }

    void genHand() throws IOException {
        for (int i = 0; i < handString.length; i++) {
            write(handString[i], handChanges[i], "tests", "subtask" + Integer.toString(subtaskCount), 0, String.format("\"hand %d\"", i + 1));
        }
    }

    void genRand(int n, int m) throws IOException {
        StringBuilder s = new StringBuilder();
        for (int i = 0; i < n; i++) {
            s.append("WB".charAt(random.nextInt(2)));
        }
        int[] q = new int[m];
        for (int i = 0; i < m; i++) {
            q[i] = random.nextInt(n) + 1;
        }
        write(s.toString(), q, "tests", "subtask" + Integer.toString(subtaskCount), 0, String.format("\"random, n=%d, m=%d\"", n, m));
    }

    void genMaxTest(int n, int m) throws IOException {
        StringBuilder s = new StringBuilder();
        for (int i = 0; i < n; i++) {
            s.append('W');
        }
        int[] q = new int[m];
        for (int i = 0; i < m;) {
            int id = random.nextInt(n);
            q[i++] = id + 1;
            if (i < m) {
                q[i++] = id + 1;
            }
        }
        write(s.toString(), q, "tests", "subtask" + Integer.toString(subtaskCount), 0, String.format("\"maxAns, n=%d, m=%d\"", n, m));
    }

    void genMorseTue(int n, int m) throws IOException {
        StringBuilder s = new StringBuilder();
        for (int i = 0; i < n; i++) {
            s.append("WB".charAt(random.nextInt(2)));
        }
        int[] q = new int[m];
        int c = 0;
        for (int i = 0; i < n; i++) {
            if ((s.charAt(i) == 'W') ^ (Integer.bitCount(i) == 0)) {
                q[c++] = i + 1;
            }
        }
        while (c < m) {
            q[c++] = 1;
        }
        write(s.toString(), q, "tests", "subtask" + Integer.toString(subtaskCount), 0, String.format("\"MorseTue, n=%d, m=%d\"", n, m));
    }

    void genSubtask(int n, int m, int points) throws IOException {
        ++subtaskCount;

        System.err.println("Subtask " + subtaskCount);
        if (subtaskCount == 1) {
            genHand();
            for (int i = 0; i < 5; i++) {
                int nn = random.nextInt(8) + 3;
                int mm = random.nextInt(5) + 1;
                genRand(nn, mm);
            }
        }
        for (int i = 0; i < 6; i++) {
            genRand(n - i, m);
        }
        genMaxTest(n, m);
        if (n <= m) {
            genMorseTue(n, m);
        }

        testsDesc.println(String.format("comment_subtask%d=\"n &lt; %d\"", subtaskCount, n));
        testsDesc.println(String.format("bonus_subtask%d=%d", subtaskCount, points));       
    }

    void run() throws IOException {
        new File("../preliminary").mkdir();
        new File("../tests").mkdir();
        testsDesc = new PrintWriter("../tests.desc");

        genPreliminary();
        genSubtask(100, 100, 25);
        genSubtask(1000, 1000, 25);
        genSubtask(200_000, 1, 25);
        genSubtask(MAX_N, MAX_N, 25);

        testsDesc.close();
    }

    public static void main(String[] args) throws IOException {
        new TestGen().run();
    }
}
